package java.lang;

import java.util.Random;

public final class StrictMath
{
    public static final double E = 2.718281828459045;
    public static final double PI = 3.141592653589793;
    
    private StrictMath() {
    }
    
    public static native double sin(double p0);
    
    public static native double cos(double p0);
    
    public static native double tan(double p0);
    
    public static native double asin(double p0);
    
    public static native double acos(double p0);
    
    public static native double atan(double p0);
    
    public static strictfp double toRadians(double n) {
        return n / 180.0 * 3.141592653589793;
    }
    
    public static strictfp double toDegrees(double n) {
        return n * 180.0 / 3.141592653589793;
    }
    
    public static native double exp(double p0);
    
    public static native double log(double p0);
    
    public static native double log10(double p0);
    
    public static native double sqrt(double p0);
    
    public static native double cbrt(double p0);
    
    public static native double IEEEremainder(double p0, double p1);
    
    public static double ceil(double n) {
        return floorOrCeil(n, -0.0, 1.0, 1.0);
    }
    
    public static double floor(double n) {
        return floorOrCeil(n, -1.0, 0.0, -1.0);
    }
    
    private static double floorOrCeil(double n, double n2, double n3, double n4) {
        int exponent = Math.getExponent(n);
        if (exponent < 0) {
            return (n == 0.0) ? n : ((n < 0.0) ? n2 : n3);
        }
        if (exponent >= 52) {
            return n;
        }
        assert exponent >= 0 && exponent <= 51;
        long doubleToRawLongBits = Double.doubleToRawLongBits(n);
        long n5 = 4503599627370495L >> exponent;
        if ((n5 & doubleToRawLongBits) == 0x0L) {
            return n;
        }
        double longBitsToDouble = Double.longBitsToDouble(doubleToRawLongBits & ~n5);
        if (n4 * n > 0.0) {
            longBitsToDouble += n4;
        }
        return longBitsToDouble;
    }
    
    public static double rint(double abs) {
        double n = 4.503599627370496E15;
        double copySign = Math.copySign(1.0, abs);
        abs = Math.abs(abs);
        if (abs < n) {
            abs = n + abs - n;
        }
        return copySign * abs;
    }
    
    public static native double atan2(double p0, double p1);
    
    public static native double pow(double p0, double p1);
    
    public static int round(float n) {
        return Math.round(n);
    }
    
    public static long round(double n) {
        return Math.round(n);
    }
    
    public static double random() {
        return StrictMath.RandomNumberGeneratorHolder.randomNumberGenerator.nextDouble();
    }
    
    public static int addExact(int n, int n2) {
        return Math.addExact(n, n2);
    }
    
    public static long addExact(long n, long n2) {
        return Math.addExact(n, n2);
    }
    
    public static int subtractExact(int n, int n2) {
        return Math.subtractExact(n, n2);
    }
    
    public static long subtractExact(long n, long n2) {
        return Math.subtractExact(n, n2);
    }
    
    public static int multiplyExact(int n, int n2) {
        return Math.multiplyExact(n, n2);
    }
    
    public static long multiplyExact(long n, long n2) {
        return Math.multiplyExact(n, n2);
    }
    
    public static int toIntExact(long n) {
        return Math.toIntExact(n);
    }
    
    public static int floorDiv(int n, int n2) {
        return Math.floorDiv(n, n2);
    }
    
    public static long floorDiv(long n, long n2) {
        return Math.floorDiv(n, n2);
    }
    
    public static int floorMod(int n, int n2) {
        return Math.floorMod(n, n2);
    }
    
    public static long floorMod(long n, long n2) {
        return Math.floorMod(n, n2);
    }
    
    public static int abs(int n) {
        return Math.abs(n);
    }
    
    public static long abs(long n) {
        return Math.abs(n);
    }
    
    public static float abs(float n) {
        return Math.abs(n);
    }
    
    public static double abs(double n) {
        return Math.abs(n);
    }
    
    public static int max(int n, int n2) {
        return Math.max(n, n2);
    }
    
    public static long max(long n, long n2) {
        return Math.max(n, n2);
    }
    
    public static float max(float n, float n2) {
        return Math.max(n, n2);
    }
    
    public static double max(double n, double n2) {
        return Math.max(n, n2);
    }
    
    public static int min(int n, int n2) {
        return Math.min(n, n2);
    }
    
    public static long min(long n, long n2) {
        return Math.min(n, n2);
    }
    
    public static float min(float n, float n2) {
        return Math.min(n, n2);
    }
    
    public static double min(double n, double n2) {
        return Math.min(n, n2);
    }
    
    public static double ulp(double n) {
        return Math.ulp(n);
    }
    
    public static float ulp(float n) {
        return Math.ulp(n);
    }
    
    public static double signum(double n) {
        return Math.signum(n);
    }
    
    public static float signum(float n) {
        return Math.signum(n);
    }
    
    public static native double sinh(double p0);
    
    public static native double cosh(double p0);
    
    public static native double tanh(double p0);
    
    public static native double hypot(double p0, double p1);
    
    public static native double expm1(double p0);
    
    public static native double log1p(double p0);
    
    public static double copySign(double n, double n2) {
        return Math.copySign(n, Double.isNaN(n2) ? 1.0 : n2);
    }
    
    public static float copySign(float n, float n2) {
        return Math.copySign(n, Float.isNaN(n2) ? 1.0f : n2);
    }
    
    public static int getExponent(float n) {
        return Math.getExponent(n);
    }
    
    public static int getExponent(double n) {
        return Math.getExponent(n);
    }
    
    public static double nextAfter(double n, double n2) {
        return Math.nextAfter(n, n2);
    }
    
    public static float nextAfter(float n, double n2) {
        return Math.nextAfter(n, n2);
    }
    
    public static double nextUp(double n) {
        return Math.nextUp(n);
    }
    
    public static float nextUp(float n) {
        return Math.nextUp(n);
    }
    
    public static double nextDown(double n) {
        return Math.nextDown(n);
    }
    
    public static float nextDown(float n) {
        return Math.nextDown(n);
    }
    
    public static double scalb(double n, int n2) {
        return Math.scalb(n, n2);
    }
    
    public static float scalb(float n, int n2) {
        return Math.scalb(n, n2);
    }
    
    private static final class RandomNumberGeneratorHolder
    {
        static final Random randomNumberGenerator;
        
        static {
            randomNumberGenerator = new Random();
        }
    }
}
